package agents

import (
	"context"
	"encoding/json"
	"screenmark/screenmark/.gen/haystack/haystack/model"
	"screenmark/screenmark/agents/client"
	"screenmark/screenmark/models"

	"github.com/charmbracelet/log"
	"github.com/google/uuid"
)

const contactPrompt = `
**Role:** You are an AI assistant specialized in processing contact information from images. Your primary function is to use the provided tools (listContacts, createContact, stopAgent) to manage contacts based on image analysis and signal when processing is complete.

**Primary Goal:** To accurately identify potential contacts in an image, check against existing contacts using the provided tools, create new contact entries when necessary (meticulously avoiding duplicates), and explicitly stop processing when finished or if no action is needed.

**Input:** You will be given an image that may contain contact information.

**Output Behavior (CRITICAL):**
* **If providing a text response:** Generate only the conversational text intended for the user in the response content. (Note: This should generally not happen in this workflow, as actions are handled by tools).
* **If using a tool:** Generate **only** the structured tool call request in the designated tool call section of the response. **Do NOT include the tool call JSON, parameters, or any description of your intention to call the tool within the main text/content response.** Your output must be strictly one or the other for a given turn: either text content OR a tool call structure.

**Core Workflow:**

1.  **Image Analysis:**
    * Carefully scan the provided image to identify and extract any visible contact details (Name, Phone Number, Email Address, Physical Address). Extract *all* available information for each potential contact.
    * **If NO potential contact information is found in the image, proceed directly to Step 5 (call stopAgent).**

2.  **Duplicate Check (Mandatory First Step if contacts found):**
    * If potential contact(s) were found in Step 1, you **must** call the listContacts tool first. **Generate only the listContacts tool call structure.**
    * Once you receive the list, compare the extracted information against the existing contacts to determine if each identified person is already present.
    * **If *all* identified potential contacts already exist in the list, proceed directly to Step 5 (call stopAgent).**

3.  **Create New Contact (Conditional):**
    * For each potential contact identified in Step 1 that your check in Step 2 confirms is *new*:
        * Call the createContact tool with *all* corresponding extracted information (name, phoneNumber, address, email). name is mandatory. **Generate only the createContact tool call structure.**
        * Process *one new contact creation per turn*. If multiple new contacts need creating, you will call createContact sequentially (one call per turn).

4.  **Handling Multiple Contacts:**
    * The workflow intrinsically handles multiple contacts by requiring a listContacts check first, followed by potential sequential createContact calls for each new individual found.

5.  **Task Completion / No Action Needed:**
    * Call the stopAgent tool **only** when one of the following conditions is met:
        * No potential contact information was found in the initial image analysis (Step 1).
        * The listContacts check confirmed that *all* potential contacts identified in the image already exist (Step 2).
        * You have successfully processed all identified contacts (i.e., performed the listContacts check and called createContact for *all* new individuals found).
    * **Generate only the stopAgent tool call structure.**

**Available Tools:**

* **listContacts**: Retrieves the existing contact list. **Must** be called first if potential contacts are found in the image, to enable duplicate checking.
* **createContact**: Adds a *new*, non-duplicate contact. Only call *after* listContacts confirms the person is new. name is mandatory.
* **stopAgent**: Signals that processing for the current image is complete (either action was taken, no action was needed, or all identified contacts already existed). Call this as the final step or when no other action is applicable based on the workflow.

**Key Instructions & Constraints:**

* **Strict Output Separation:** **Never mix text content and tool call structures in the same response content. Generate ONLY the tool call when using a tool.**
* **Tool Order:** listContacts (if contacts found) -> [createContact (if new contacts exist, potentially multiple calls)] -> stopAgent. OR stopAgent directly if no contacts found or all contacts exist.
* **Accuracy & Completeness:** Strive for accurate extraction and provide all available details when calling createContact.
* **No Duplicates:** The core purpose of the listContacts check is to prevent duplicate entries.
* **Use Provided Tools Only:** Only use listContacts, createContact, and stopAgent.
`

const contactTools = `
[
  {
    "type": "function",
    "function": {
      "name": "listContacts",
      "description": "Retrieves the complete list of the user's currently saved contacts (e.g., names, phone numbers, emails if available in the stored data). This tool is essential and **must** be called *before* attempting to create a new contact if potential contact info is found in the image, to check if the person already exists and prevent duplicate entries.",
      "parameters": {
        "type": "object",
        "properties": {},
        "required": []
      }
    }
  },
  {
    "type": "function",
    "function": {
      "name": "createContact",
      "description": "Saves a new contact to the user's contact list. Only use this function **after** confirming the contact does not already exist by checking the output of listContacts. Provide all available extracted information for the new contact. Process one new contact per call.",
      "parameters": {
        "type": "object",
        "properties": {
          "name": {
            "type": "string",
            "description": "The full name of the person being added as a contact. This field is mandatory."
          },
          "phoneNumber": {
            "type": "string",
            "description": "The contact's primary phone number, including area or country code if available. Provide this if extracted from the image."
          },
          "address": {
            "type": "string",
            "description": "The complete physical mailing address of the contact (e.g., street number, street name, city, state/province, postal code, country). Provide this if extracted from the image."
          },
          "email": {
            "type": "string",
            "description": "The contact's primary email address. Provide this if extracted from the image."
          }
        },
        "required": ["name"]
      }
    }
  },
  {
    "type": "function",
    "function": {
      "name": "stopAgent",
      "description": "Use this tool to signal that the contact processing for the current image is complete. Call this *only* when: 1) No contact info was found initially, OR 2) All found contacts were confirmed to already exist after calling listContacts, OR 3) All necessary createContact calls for new individuals have been completed.",
      "parameters": {
        "type": "object",
        "properties": {},
        "required": []
      }
    }
  }
]
`

type listContactsArguments struct{}
type createContactsArguments struct {
	Name        string  `json:"name"`
	PhoneNumber *string `json:"phoneNumber"`
	Address     *string `json:"address"`
	Email       *string `json:"email"`
}
type linkContactArguments struct {
	ContactID string `json:"contactId"`
}

func NewContactAgent(log *log.Logger, contactModel models.ContactModel) client.AgentClient {
	agentClient := client.CreateAgentClient(client.CreateAgentClientOptions{
		SystemPrompt: contactPrompt,
		JsonTools:    contactTools,
		Log:          log,
		EndToolCall:  "stopAgent",
	})

	agentClient.ToolHandler.AddTool("listContacts", func(info client.ToolHandlerInfo, args string, call client.ToolCall) (any, error) {
		return contactModel.List(context.Background(), info.UserId)
	})

	agentClient.ToolHandler.AddTool("createContact", func(info client.ToolHandlerInfo, _args string, call client.ToolCall) (any, error) {
		args := createContactsArguments{}
		err := json.Unmarshal([]byte(_args), &args)
		if err != nil {
			return model.Contacts{}, err
		}

		ctx := context.Background()

		contact, err := contactModel.Save(ctx, info.UserId, model.Contacts{
			Name:        args.Name,
			PhoneNumber: args.PhoneNumber,
			Email:       args.Email,
		})

		if err != nil {
			return model.Contacts{}, err
		}

		_, err = contactModel.SaveToImage(ctx, info.ImageId, contact.ID)
		if err != nil {
			return model.Contacts{}, err
		}

		return contact, nil
	})

	agentClient.ToolHandler.AddTool("linkContact", func(info client.ToolHandlerInfo, _args string, call client.ToolCall) (any, error) {
		args := linkContactArguments{}
		err := json.Unmarshal([]byte(_args), &args)
		if err != nil {
			return "", err
		}

		ctx := context.Background()

		contactUuid, err := uuid.Parse(args.ContactID)
		if err != nil {
			return "", err
		}

		_, err = contactModel.SaveToImage(ctx, info.ImageId, contactUuid)
		if err != nil {
			return "", err
		}

		return "Saved", nil
	})

	return agentClient
}
